# © 2019 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import models, fields, api
from odoo.exceptions import ValidationError

PERCENT = [
    (0, 0),
    (20, 20),
    (40, 40),
    (60, 60),
    (80, 80),
    (100, 100)]


class ScopPartner(models.Model):
    _inherit = "res.partner"

    # Processus d'incubation
    incub_status = fields.Selection([
        ('1_prospection', "Phase de prospection et de candidature"),
        ('2_pre-incubation', 'Phase de pré-incubation'),
        ('3_incubation', "Phase d'incubation"),
        ('4_sortie_incub', "Sortie d'incubation"),
        ('9_abandonne', 'Projet abandonné')],
        track_visibility='onchange',
        string='Statut incubation',
        index=True)
    incub_out_mode = fields.Selection([
        ('1_non_def', "Non définie"),
        ('2_non_coop', "Non coopérative"),
        ('3_coop_non_adh', "Coopérative non adhérente"),
        ('4_coop_adh', "Coopérative projet d'adhésion")],
        default='1_non_def',
        track_visibility='onchange',
        string='Sortie incubation',
        index=True)
    incub_out_done = fields.Boolean(
        string='Société créée',
        default=False)
    incub_active = fields.Boolean(
        string="Projet actif",
        default=True)
    # Données projet
    incub_prescription_id = fields.Many2one(
        'cgscop_incub.prescription',
        string='Canal de prescription',
        required=False,
        track_visibility='onchange')
    incub_date_candidature = fields.Date(
        string="Date de candidature",
        track_visibility='onchange')
    incub_date_prospection = fields.Date(
        string="Date de début du dossier",
        track_visibility='onchange')
    incub_delegate_id = fields.Many2one(
        'res.users',
        string='Délégué incubation',
        on_delete='restrict',
        track_visibility='onchange')
    incub_thematique_ids = fields.Many2many(
        comodel_name='cgscop_incub.thematique',
        column1='partner_id',
        column2='thematique_id',
        string='Thematique du projet',
        required=False,
        track_visibility='onchange')
    incub_typologie_porteur_id = fields.Many2one(
        'cgscop_incub.typologie_porteur',
        string='Typologie du porteur de projet',
        required=False,
        track_visibility='onchange')
    incub_projet_description = fields.Text(
        string='Description du projet',
        required=False,
        track_visibility='onchange')
    incub_nb_assos = fields.Integer(
        string="Nombre d'associés au projet",
        required=False,
        track_visibility='onchange')
    incub_sensibilite_coop = fields.Selection(
        selection=PERCENT,
        string='Sensibilté coopérative',
        required=False,
        track_visibility='onchange')
    incub_percent_realisation = fields.Selection(
        selection=PERCENT,
        string='Probabilité de réalisation',
        required=False,
        track_visibility='onchange')
    incub_fiche_entree = fields.Boolean(
        string="Fiche d'entrée",
        required=False,
        track_visibility='onchange')
    incub_fiche_sortie = fields.Boolean(
        string="Fiche de sortie",
        required=False,
        track_visibility='onchange')
    incub_convention = fields.Boolean(
        string="Convention d'accompagnement",
        required=False,
        track_visibility='onchange')
    incub_preincub_bypass = fields.Boolean(
        string="Pas de pré-incubation",
        required=False,
        track_visibility='onchange')
    incub_incubation_bypass = fields.Boolean(
        string="Pas d'incubation",
        required=False,
        track_visibility='onchange')
    incub_preincub_deb = fields.Date(
        string="Date d'entrée pré-incubation",
        track_visibility='onchange')
    incub_preincub_fin = fields.Date(
        string="Date de sortie pré-incubation",
        track_visibility='onchange')
    incub_parcours_preincub = fields.Many2one(
        'cgscop_incub.parcours_preincub',
        string='Parcours pré-incubation',
        required=False,
        track_visibility='onchange')
    incub_statut_prefig = fields.Many2one(
        'cgscop_incub.statut_prefig',
        string='Statuts de préfiguration',
        required=False,
        track_visibility='onchange')
    incub_prefig_date = fields.Date(
        string="Date de création préfiguration",
        track_visibility='onchange')
    incub_inter_bilan_date = fields.Date(
        string="Date de bilan intermédiaire",
        track_visibility='onchange')
    incub_incubation_deb = fields.Date(
        string="Date d'entrée incubation",
        track_visibility='onchange')
    incub_incubation_fin = fields.Date(
        string="Date de sortie incubation",
        track_visibility='onchange')
    incub_expertise_ids = fields.One2many(
        comodel_name='cgscop_incub.expertise_projet',
        inverse_name='partner_id',
        string='Expertises')
    incub_preincub_mt_exp = fields.Float(
        string="Montant expertises pre-incub",
        compute="_compute_preincub_montant",
        store=True)
    incub_incubation_mt_exp = fields.Float(
        string="Montant expertises incubation",
        compute="_compute_incubation_montant",
        store=True)
    incub_financement_ids = fields.One2many(
        comodel_name='cgscop_incub.financement_projet',
        inverse_name='partner_id',
        string='Financements')
    incub_financement_mt_dmd = fields.Float(
        string="Financement montant demandé",
        compute="_compute_financement_mt",
        store=True)
    incub_financement_mt_recu = fields.Float(
        string="Financement montant recu",
        compute="_compute_financement_mt",
        store=True)
    incub_sortie_notes = fields.Text(
        string='Commentaires fin de projet',
        required=False,
        track_visibility='onchange')
    incub_motif_abandon_id = fields.Many2one(
        'cgscop_incub.motif_abandon',
        string="Motif d'abandon",
        required=False,
        track_visibility='onchange')
    incub_out_is_cae = fields.Boolean(
        string="A rejoint une CAE existante",
        required=False,
        track_visibility='onchange')
    incub_out_cae_name = fields.Char(
        string='Nom de la CAE',
        required=False,
        track_visibility='onchange')

    # ------------------------------------------------------
    # Computed
    # ------------------------------------------------------
    @api.multi
    @api.depends('incub_expertise_ids')
    def _compute_preincub_montant(self):
        for partner in self:
            xps = partner.incub_expertise_ids.search([
                ('partner_id', '=', partner.id),
                ('incub_phase', '=', '2_pre-incubation')])
            if xps:
                for xp in xps:
                    partner.incub_preincub_mt_exp = (
                        partner.incub_preincub_mt_exp + xp.montant)

    @api.depends('incub_expertise_ids')
    @api.multi
    def _compute_incubation_montant(self):
        for partner in self:
            xps = partner.incub_expertise_ids.search([
                ('partner_id', '=', partner.id),
                ('incub_phase', '=', '3_incubation')],)
            if xps:
                for xp in xps:
                    partner.incub_incubation_mt_exp = (
                        partner.incub_incubation_mt_exp + xp.montant)

    @api.depends('incub_financement_ids')
    @api.multi
    def _compute_financement_mt(self):
        for partner in self:
            fis = partner.incub_financement_ids.search(
                [['partner_id', '=', partner.id]])
            if fis:
                for fi in fis:
                    partner.incub_financement_mt_dmd = (
                        partner.incub_financement_mt_dmd + fi.montant_dmd)
                    partner.incub_financement_mt_recu = (
                        partner.incub_financement_mt_recu + fi.montant_recu)

    # ------------------------------------------------------
    # Constains
    # ------------------------------------------------------
    @api.constrains('incub_preincub_deb', 'incub_preincub_fin')
    @api.multi
    def _preincub_date_constrains(self):
        for rec in self:
            if rec.incub_preincub_fin and rec.incub_preincub_deb:
                if rec.incub_preincub_fin < rec.incub_preincub_deb:
                    raise ValidationError("La date de fin de pré-incubation \
                        doit être supérieure à la date de début")

    @api.constrains('incub_incubation_deb', 'incub_incubation_fin')
    @api.multi
    def _incubation_date_constrains(self):
        for rec in self:
            if rec.incub_incubation_fin and rec.incub_incubation_deb:
                if rec.incub_incubation_fin < rec.incub_incubation_deb:
                    raise ValidationError("La date de fin d'incubation doit \
                        être supérieure à la date de début")

    @api.constrains('incub_preincub_fin', 'incub_incubation_deb')
    @api.multi
    def _incub_date_constrains(self):
        for rec in self:
            if rec.incub_preincub_fin and rec.incub_incubation_deb:
                if rec.incub_incubation_deb < rec.incub_preincub_fin:
                    raise ValidationError("La date de début d'incubation doit \
                        être supérieure à la date de fin de pre-incubation")

    # ------------------------------------------------------
    # Button
    # ------------------------------------------------------
    @api.multi
    def incub_non_coop(self):
        for rec in self:
            rec.is_cooperative = False
            rec.incub_out_done = True

    @api.multi
    def incub_coop_non_adh(self):
        for rec in self:
            rec.is_cooperative = True
            rec.project_status = '6_suivi'
            rec.incub_out_done = True

    @api.multi
    def incub_coop_adh(self):
        for rec in self:
            rec.is_cooperative = True
            rec.project_status = '2_pre-diagnostic'
            rec.incub_out_done = True

    @api.multi
    def toggle_incub_active(self):
        for rec in self:
            rec.incub_active = not rec.incub_active
